/***************************************************************************
 *   Copyright (C) 2004-2005 by Alexander Saal                                  *
 *   alex.saal@gmx.de                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


/****************
 * self include *
 ****************/
#include "dosboxfe.moc"
#include "dosboxfe.h"
#include "dosboxfe_profile.h"
#include "gamedownload/dosboxfe_download.h"
#include "searchengine/doxboxfe_wizard.h"
#include "updatewizard/dosboxfe_updatewizard.h"

#include "updatewizard/update.h"  // for update my programm
#include "gprofile.h"  // for create a game profile, in this header file is the declaration for KConfig

/*
  system include
*/
#include <string.h>
#include <stdio.h>
#include <unistd.h>

/**************************
 * Min. KDE 3.x.x include *
 **************************/
#include <kconfig.h>
#include <kapp.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <kglobal.h>
#include <klocale.h>

/*************************
 * Min. Qt 3.x.x include *
 *************************/
#include <qdir.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qprocess.h>  // this is for dosbox and dosbox cvs
#include <qstringlist.h>
#include <qurloperator.h> // for download the update
#include <qnetworkprotocol.h> // for network connection
#include <qnetwork.h>  // for network connection
#include <qstring.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qlcdnumber.h>
#include <qslider.h>
#include <qtextedit.h>
#include <qprogressbar.h> // for stauts information about the download
#include <qwidget.h>
#include <qframe.h>
#include <qmenubar.h>
#include <qlabel.h>
#include <qcursor.h>
#include <qpopupmenu.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qlibrary.h>



static const char version[] = "0.0.6-0.63"; // Current Version
unsigned int sleep(unsigned int seconds); // sleep my app sleep for n seconds ;)

DosBoxFE::DosBoxFE(QWidget* parent, const char* name, bool modal, WFlags fl)
                : DBFE_MainDlg(parent,name, modal,fl)
{
        // Network init.
        qInitNetworkProtocols();     // Init network, is requiered by update

        LBox_OutPut->insertItem("DBoxFE is started\n");  // debug message

        TextLabel_AvailableVersion->setText( version );  // Set the version of DBoxFE
        btt_Download_Update->setEnabled(false); // set enabled false

        KConfig *confAutoexec( kapp->config() );
        confAutoexec = new KConfig( "autoexec.dbfe");   // This load the dir into the AutoexecBox
        confAutoexec->setGroup( "Mount Dir" );
        ListBox_MountDir->insertStringList( confAutoexec->readListEntry( "dir", ';' ));

        KConfig *confGameProfile( kapp->config() );   // This Load the entry of Games
        confGameProfile = new KConfig( "game_profile.dbfe" );
        confGameProfile->setGroup( "Game Profile" );
        ListBox_GameProfile->insertStringList( confGameProfile->readListEntry( "Games", ':' ));

        KConfig *confBoxFErc( kapp->config() );
        confBoxFErc = new KConfig( "dboxferc" );   // This load the saved version to you use (cvs or current )
        confBoxFErc->setGroup( "DOSBox Version" );
        RadioButton_DBox_Current->setChecked( confBoxFErc->readBoolEntry( "Version_Cuurent", RadioButton_DBox_Current->isChecked() ) );
        RadioButton_DBox_CVS->setChecked( confBoxFErc->readBoolEntry( "Version_CVS", RadioButton_DBox_CVS->isChecked() ) );

        confBoxFErc->setGroup( "DOSBox_Binary" );
        LineEdit_DOSBox_Binary_Current->setText( confBoxFErc->readEntry( "Stabel binary", LineEdit_DOSBox_Binary_Current->text() ) );
        LineEdit_DOSBox_Binary_CVS->setText( confBoxFErc->readEntry( "CVS binary", LineEdit_DOSBox_Binary_CVS->text() ) );

        confBoxFErc->setGroup( "Splashscreen" );
        chBox_DisableSplash->setChecked( confBoxFErc->readBoolEntry("Splash", chBox_DisableSplash->isChecked() ) );

        // This load the gamefiles from file into the listbox
        KConfig *confGameFiles( kapp->config() );
        confGameFiles = new KConfig( "gamefiles.dbfe" );
        confGameFiles->setGroup( "Gamesfiles" );
        ListBox_Game_File->insertStringList( confGameFiles->readListEntry( "Games", ':' ));

        // This load the savegamedir from file into the listbox
        KConfig *saveGameDir( kapp->config() );
        saveGameDir = new KConfig( "gamefiles.dbfe");
        saveGameDir->setGroup( "Savegame Dir" );
        ListBox_Savegame_Dir->insertStringList( saveGameDir->readListEntry( "SaveGameDir", ':'));

}

DosBoxFE::~DosBoxFE()
{}
/******************************************************************/// this removed a selected game profile, if the listbox empty, then display a message :)
void DosBoxFE::slot_Remove_AutoexecDir()
{
        if( ListBox_MountDir->currentText().isEmpty() ) {
                KMessageBox::information( this, i18n("Please select a dir in order to delete it!") );
        } else {
                delete ListBox_MountDir->selectedItem();
                LBox_OutPut->insertItem( i18n("Mount Directory -> '") + ListBox_MountDir->currentText() +  i18n("' was deleted") );
        }
}
/******************************************************************/// this removed a selected game profile, if the listbox empty, then display a message :)
void DosBoxFE::slot_Remove_GameProfile()
{
        if( ListBox_GameProfile->currentText().isEmpty() ) {
                KMessageBox::information( this, i18n("Please select profiles in order to delete it!") );
        } else {
                delete ListBox_GameProfile->selectedItem();
                LBox_OutPut->insertItem( i18n("Game Profile -> was deleted") );
        }
}

/**********************************************************************************************/// open a game download diaolg, not complet and was disabled
void DosBoxFE::slot_Game_Download()
{
        DosBoxFE_Download *dbfe_down = new DosBoxFE_Download();
        dbfe_down->show();
}

/***************************************************************************************************/// this make a entry in the listbox
void DosBoxFE::slot_Add_GameProfile()
{
        DosBoxFE_Profile dbfe_profile(this);
        if ( dbfe_profile.exec() == QDialog::Accepted ) {
                ListBox_GameProfile->insertItem( dbfe_profile.LineEdit_ProfileName->text() );
        }
}

/****************************************************************/// click in the listbox and the conig file is readed ( a simpel kconfig code but genial :)
void DosBoxFE::slot_Open_Config()
{
        KConfig *config( kapp->config() );
        if ( ListBox_GameProfile->currentText().isEmpty() ) {
                KMessageBox::information( this, i18n("Please select a Game Profile!") );
        } else {
                if ( RadioButton_DBox_Current->isChecked() ) {
                        config = new KConfig( CVS_Version ); //for v 0.63 (DOSBox)

                        // read "sdl" config
                        config->setGroup( "sdl" );
                        CheckBox_SDL_Full_Screen->setChecked( config->readBoolEntry( "fullscreen",CheckBox_SDL_Full_Screen->isChecked() ) );
                        CheckBox_SDL_Full_Double->setChecked( config->readBoolEntry( "fulldouble",CheckBox_SDL_Full_Double->isChecked() ) );
                        CheckBox_SDL_Full_Fixed->setChecked( config->readBoolEntry( "fullfixed",CheckBox_SDL_Full_Fixed->isChecked() ) );
                        ComboBox_SDL_Resoulution_Width->setCurrentText( config->readEntry( "fullwidth",ComboBox_SDL_Resoulution_Width->currentText() ) );
                        ComboBox_SDL_Resoulution_Height->setCurrentText( config->readEntry( "fullheight",ComboBox_SDL_Resoulution_Height->currentText() ) );
                        ComboBox_SDL_Output->setCurrentText( config->readEntry( "output", ComboBox_SDL_Output->currentText() ) );
                        LCDNumber_SDL_hwScale->display( config->readNumEntry( "hwscale", LCDNumber_SDL_hwScale->intValue() ) );
                        LCDNumber_SDL_Sensitivity->display( config->readNumEntry( "sensitivity",LCDNumber_SDL_Sensitivity->intValue() ) );
                        CheckBox_SDL_Autolock->setChecked( config->readBoolEntry( "autolock", CheckBox_SDL_Autolock->isChecked() ) );
                        CheckBox_SDL_WaitOnError->setChecked( config->readBoolEntry( "waitonerror",CheckBox_SDL_WaitOnError->isChecked() ) );
                        ComboBox_SDL_Priority_Focus_Unfocused->setCurrentText( config->readEntry( "priority",ComboBox_SDL_Priority_Focus_Unfocused->currentText() ) );

                        // read "dosbox" config
                        config->setGroup( "dosbox" );
                        LineEdit_DB_Language->setText( config->readEntry( "language", LineEdit_DB_Language->text() ) );
                        ComboBox_DB_Machine->setCurrentText( config->readEntry( "machine", ComboBox_DB_Machine->currentText() ) );
                        ComboBox_DB_MemSize->setCurrentText( config->readEntry( "memsize", ComboBox_DB_MemSize->currentText() ) );
                        ComboBox_DB_Captures->setCurrentText( config->readEntry( "captures", ComboBox_DB_Captures->currentText() ) );

                        // read "render" config
                        config->setGroup( "render" );
                        LCDNumber_Render_Frameskip->display( config->readNumEntry( "frameskip",LCDNumber_Render_Frameskip->intValue() ) );
                        LineEdit_Render_SnapDir->setText( config->readEntry( "snapdir", LineEdit_Render_SnapDir->text() ) );
                        ComboBox_Render_Scaler->setCurrentText( config->readEntry( "scaler", ComboBox_Render_Scaler->currentText() ) );
                        CheckBox_Render_Aspect->setChecked( config->readBoolEntry( "aspect", CheckBox_Render_Aspect->isChecked() ) );

                        // read "cpu" config
                        config->setGroup( "cpu" );
                        ComboBox_CPU_Core->setCurrentText( config->readEntry( "core", ComboBox_CPU_Core->currentText() ) );
                        ComboBox_CPU_Cycles->setCurrentText( config->readEntry( "cycles", ComboBox_CPU_Cycles->currentText() ) );
                        ComboBox_CPU_Cycleup->setCurrentText( config->readEntry( "cycleup", ComboBox_CPU_Cycleup->currentText() ) );
                        ComboBox_CPU_Cycledown->setCurrentText( config->readEntry( "cycledown",ComboBox_CPU_Cycledown->currentText() ) );

                        // read sound config "mixer"
                        config->setGroup( "mixer" );
                        CheckBox_Mixer_NoSound->setChecked( config->readBoolEntry( "nosound", CheckBox_Mixer_NoSound->isChecked() ) );
                        ComboBox_Mixer_Rate->setCurrentText( config->readEntry( "rate", ComboBox_Mixer_Rate->currentText() ) );
                        ComboBox_Mix_BlockSize->setCurrentText( config->readEntry( "blocksize",ComboBox_Mix_BlockSize->currentText() ) );
                        LineEdit_Mixer_WaveDir->setText( config->readEntry( "wavedir", LineEdit_Mixer_WaveDir->text() ) );

                        // read sound config "midi"
                        config->setGroup( "midi" );
                        CheckBox_Midi_MPU401->setChecked( config->readBoolEntry( "mpu401", CheckBox_Midi_MPU401->isChecked() ) );
                        ComboBox_Midi_Device->setCurrentText( config->readEntry( "device", ComboBox_Midi_Device->currentText() ) );
                        LineEdit_Midi_Config->setText( config->readEntry( "config", LineEdit_Midi_Config->text() ) );
                        CheckBox_Midi_Intelligent->setChecked( config->readBoolEntry( "intelligent",CheckBox_Midi_Intelligent->isChecked() ) );

                        // read sound config "sblaster"
                        config->setGroup( "sblaster" );
                        ComboBox_SBlaster_Type_CVS->setCurrentText( config->readEntry( "type",ComboBox_SBlaster_Type_CVS->currentText() ) );
                        ComboBox_SBlaster_Base_CVS->setCurrentText( config->readEntry( "base",ComboBox_SBlaster_Base_CVS->currentText() ) );
                        ComboBox_SBlaster_IRQ_CVS->setCurrentText( config->readEntry( "irq",ComboBox_SBlaster_IRQ_CVS->currentText() ) );
                        ComboBox_SBlaster_DMA_CVS->setCurrentText( config->readEntry( "dma",ComboBox_SBlaster_DMA_CVS->currentText() ) );
                        ComboBox_SBlaster_HDMA_CVS->setCurrentText( config->readEntry( "hdma",ComboBox_SBlaster_HDMA_CVS->currentText() ) );
                        CheckBox_SBlaster_Mixer_CVS->setChecked( config->readBoolEntry( "mixer",CheckBox_SBlaster_Mixer_CVS->isChecked() ) );
                        ComboBox_SBlaster_OPLRate_CVS->setCurrentText( config->readEntry( "oplrate",ComboBox_SBlaster_OPLRate_CVS->currentText() ) );
                        ComboBox_Sblaster_OPLMode_CVS->setCurrentText( config->readEntry( "oplmode",ComboBox_Sblaster_OPLMode_CVS->currentText() ) );

                        // read sound config "gus"
                        config->setGroup( "gus" );
                        CheckBox_GravisUltraSound->setChecked( config->readBoolEntry( "gus", CheckBox_GravisUltraSound->isChecked() ) );
                        ComboBox_GUS_Rate->setCurrentText( config->readEntry( "rate", ComboBox_GUS_Rate->currentText() ) );
                        ComboBox_GUS_Base->setCurrentText( config->readEntry( "base", ComboBox_GUS_Base->currentText() ) );
                        ComboBox_GUS_IRQ1->setCurrentText( config->readEntry( "irq1", ComboBox_GUS_IRQ1->currentText() ) );
                        ComboBox_GUS_IRQ2->setCurrentText( config->readEntry( "irq2", ComboBox_GUS_IRQ2->currentText() ) );
                        ComboBox_GUS_DMA1->setCurrentText( config->readEntry( "dma1", ComboBox_GUS_DMA1->currentText() ) );
                        ComboBox_GUS_DMA2->setCurrentText( config->readEntry( "dma2", ComboBox_GUS_DMA2->currentText() ) );
                        LineEdit_GUS_UltraDir->setText( config->readEntry( "ultradir", LineEdit_GUS_UltraDir->text() ) );

                        // read sound config "speaker"
                        config->setGroup( "speaker" );
                        CheckBox_PC_Speaker->setChecked( config->readBoolEntry( "pcspeaker", CheckBox_PC_Speaker->isChecked() ) );
                        ComboBox_PC_SpeakerRate->setCurrentText( config->readEntry( "pcrate", ComboBox_PC_SpeakerRate->currentText() ) );
                        CheckBox_PC_Tandy->setChecked( config->readBoolEntry( "tandy", CheckBox_PC_Tandy->isChecked() ) );
                        ComboBox_PC_TandyRate->setCurrentText( config->readEntry( "tandyrate", ComboBox_PC_TandyRate->currentText() ) );
                        CheckBox_PC_Disney->setChecked( config->readBoolEntry( "disney", CheckBox_PC_Disney->isChecked() ) );

                        config->setGroup( "modem" );
                        config->writeEntry( "modem", CheckBox_Modem->isChecked() );
                        config->writeEntry( "comport", ComboBox_Modem_ComPort->currentText() );
                        config->writeEntry( "listenport", ComboBox_Modem_ListenPort->currentText() );

                        config->setGroup( "ipx" );
                        CheckBox_IPX->setChecked( config->readBoolEntry( "ipx", CheckBox_IPX->isChecked() ) );

                        LBox_OutPut->insertItem( i18n("Profile -> <") + ListBox_GameProfile->currentText() + i18n("> loaded") );
                } else {
                        KMessageBox::information( this, i18n("Please select a DOSBox version!") );
                }
        }

}

/***************************************************************************************************/// Save Configfile from Gameprofile and another entries
void DosBoxFE::slot_Save_Config()
{
        QStringList sList, sList_MountDir, sList_Priority;

        KConfig *config( kapp->config() );
        if ( ListBox_GameProfile->currentText().isEmpty() ) {
                KMessageBox::information( this, "No Game Profile found!");
        } else {
                if ( RadioButton_DBox_Current->isChecked() ) {
                        config = new KConfig( CVS_Version ); //for v 0.63 (DOSBox)
                        // SDL configuratin
                        config->setGroup( "sdl" );
                        config->writeEntry( "fullscreen", CheckBox_SDL_Full_Screen->isChecked() );
                        config->writeEntry( "fulldouble", CheckBox_SDL_Full_Double->isChecked() );
                        config->writeEntry( "fullfixed", CheckBox_SDL_Full_Fixed->isChecked() );
                        config->writeEntry( "fullwidth", ComboBox_SDL_Resoulution_Width->currentText() );
                        config->writeEntry( "fullheight", ComboBox_SDL_Resoulution_Height->currentText() );
                        config->writeEntry( "output", ComboBox_SDL_Output->currentText() );
                        config->writeEntry( "hwscale", LCDNumber_SDL_hwScale->intValue() );
                        config->writeEntry( "sensitivity", LCDNumber_SDL_Sensitivity->intValue() );
                        config->writeEntry( "autolock", CheckBox_SDL_Autolock->isChecked() );
                        config->writeEntry( "waitonerror", CheckBox_SDL_WaitOnError->isChecked() );
                        config->writeEntry( "priority",ComboBox_SDL_Priority_Focus_Unfocused->currentText() );

                        // DosBox configuratin
                        config->setGroup( "dosbox" );
                        config->writeEntry( "language", LineEdit_DB_Language->text() );
                        config->writeEntry( "machine", ComboBox_DB_Machine->currentText() );
                        config->writeEntry( "memsize", ComboBox_DB_MemSize->currentText() );
                        config->writeEntry( "captures", ComboBox_DB_Captures->currentText() );


                        // Render configuratin
                        config->setGroup( "render" );
                        config->writeEntry( "frameskip",LCDNumber_Render_Frameskip->intValue() );
                        config->writeEntry( "snapdir", LineEdit_Render_SnapDir->text() );
                        config->writeEntry( "scaler", ComboBox_Render_Scaler->currentText() );
                        config->writeEntry( "aspect", CheckBox_Render_Aspect->isChecked() );

                        // CPU configuratin
                        config->setGroup( "cpu" );
                        config->writeEntry( "core", ComboBox_CPU_Core->currentText() );
                        config->writeEntry( "cycles", ComboBox_CPU_Cycles->currentText() );
                        config->writeEntry( "cycleup", ComboBox_CPU_Cycleup->currentText() );
                        config->writeEntry( "cycledown", ComboBox_CPU_Cycledown->currentText() );

                        // Sound configuration
                        config->setGroup( "mixer" );
                        config->writeEntry( "nosound", CheckBox_Mixer_NoSound->isChecked() );
                        config->writeEntry( "rate", ComboBox_Mixer_Rate->currentText() );
                        config->writeEntry( "blocksize", ComboBox_Mix_BlockSize->currentText() );
                        config->writeEntry( "wavedir", LineEdit_Mixer_WaveDir->text() );

                        config->setGroup( "midi" );
                        config->writeEntry( "mpu401", CheckBox_Midi_MPU401->isChecked() );
                        config->writeEntry( "device", ComboBox_Midi_Device->currentText() );
                        config->writeEntry( "config", LineEdit_Midi_Config->text() );
                        config->writeEntry( "intelligent", CheckBox_Midi_Intelligent->isChecked() );

                        config->setGroup( "sblaster" );
                        config->writeEntry( "type", ComboBox_SBlaster_Type_CVS->currentText() );
                        config->writeEntry( "base", ComboBox_SBlaster_Base_CVS->currentText() );
                        config->writeEntry( "irq", ComboBox_SBlaster_IRQ_CVS->currentText() );
                        config->writeEntry( "dma", ComboBox_SBlaster_DMA_CVS->currentText() );
                        config->writeEntry( "hdma", ComboBox_SBlaster_HDMA_CVS->currentText() );
                        config->writeEntry( "mixer", CheckBox_SBlaster_Mixer_CVS->isChecked() );
                        config->writeEntry( "oplrate", ComboBox_SBlaster_OPLRate_CVS->currentText() );
                        config->writeEntry( "oplmode", ComboBox_Sblaster_OPLMode_CVS->currentText() );

                        config->setGroup( "gus" );
                        config->writeEntry( "gus", CheckBox_GravisUltraSound->isChecked() );
                        config->writeEntry( "base", ComboBox_GUS_Base->currentText() );
                        config->writeEntry( "rate", ComboBox_GUS_Rate->currentText() );
                        config->writeEntry( "irq1", ComboBox_GUS_IRQ1->currentText() );
                        config->writeEntry( "irq2", ComboBox_GUS_IRQ2->currentText() );
                        config->writeEntry( "dma1", ComboBox_GUS_DMA1->currentText() );
                        config->writeEntry( "dma2", ComboBox_GUS_DMA2->currentText() );
                        config->writeEntry( "ultradir", LineEdit_GUS_UltraDir->text() );

                        config->setGroup( "speaker" );
                        config->writeEntry( "pcspeaker", CheckBox_PC_Speaker->isChecked() );
                        config->writeEntry( "pcrate", ComboBox_PC_SpeakerRate->currentText() );
                        config->writeEntry( "tandy", CheckBox_PC_Tandy->isChecked() );
                        config->writeEntry( "tandyrate", ComboBox_PC_TandyRate->currentText() );
                        config->writeEntry( "disney", CheckBox_PC_Disney->isChecked() );

                        // Misc ( DOS, Modem and Autoexc )
                        config->setGroup( "dos" );
                        config->writeEntry( "xms", CheckBox_Dos_XMS->isChecked() );
                        config->writeEntry( "ems", CheckBox_Dos_EMS->isChecked() );

                        config->setGroup( "modem" );
                        config->writeEntry( "modem", CheckBox_Modem->isChecked() );
                        config->writeEntry( "comport", ComboBox_Modem_ComPort->currentText() );
                        config->writeEntry( "listenport", ComboBox_Modem_ListenPort->currentText() );

                        config->setGroup( "ipx" );
                        config->writeEntry( "ipx", CheckBox_IPX->isChecked() );

                        config->sync();

                        // Save ListBox entry
                        for( unsigned int q = 0; q < ListBox_MountDir->count(); q++ ) {
                                QString fn;
                                fn = QDir::homeDirPath();
                                fn.append( "/.kde/share/config/dosbox_" + ListBox_GameProfile->currentText() + "_.conf_curr" );

                                QFile f( fn );
                                if ( !f.open( IO_Append|IO_WriteOnly ) )
                                        return;

                                QTextStream t( &f );
                                t << "\n[autoexec]\n" + ListBox_MountDir->item( q )->text() + "\n";

                                f.close();
                        }
                        // Save ListBox entry
                        LBox_OutPut->insertItem( i18n("Profile -> <") + ListBox_GameProfile->currentText() + i18n("> save complet") );
                } else {
                        KMessageBox::information( this, i18n("Please select a DOSBox version!") );
                }
        }

        config = new KConfig( "dboxferc" );
        config->setGroup( "DOSBox Version" );
        config->writeEntry( "Version_Cuurent",RadioButton_DBox_Current->isChecked() );
        config->writeEntry( "Version_CVS", RadioButton_DBox_CVS->isChecked() );

        config->setGroup( "DOSBox_Binary" );
        config->writeEntry( "Stabel binary", LineEdit_DOSBox_Binary_Current->text() );
        config->writeEntry( "CVS binary", LineEdit_DOSBox_Binary_CVS->text() );

        config->setGroup( "Splashscreen" );
        config->writeEntry( "Splash", chBox_DisableSplash->isChecked() );
        config->sync();

        // this make a seperate gameprofile file and a seperate mount directory

        // For Games
        for( unsigned int l = 0; l < ListBox_GameProfile->count(); l++ ) {
                sList.append( ListBox_GameProfile->item( l )->text() );
        }

        config = new KConfig( "game_profile.dbfe");
        config->setGroup( "Game Profile" );
        config->writeEntry( "Games", sList,':' );
        config->sync();

        // For Autoexec ( mounted dir )
        for( unsigned int a = 0; a < ListBox_MountDir->count(); a++ ) {
                sList_MountDir.append( ListBox_MountDir->item( a )->text() );
        }

        config = new KConfig( "autoexec.dbfe");
        config->setGroup( "Mount Dir" );
        config->writeEntry( "dir", sList_MountDir,';' );
        config->sync();
}

/***********************************************************************************************************/// this start dosbox with config file
void DosBoxFE::slot_Start_DosBox()
{
        QString db_dir, db_dir_cvs, test_build;

        //db_dir = QDir::homeDirPath();
        //db_dir.append( "/.kde/share/config/dosbox_" + ListBox_GameProfile->currentText() + "_.conf_current" );

        db_dir_cvs = QDir::homeDirPath();
        db_dir_cvs.append( "/.kde/share/config/dosbox_" + ListBox_GameProfile->currentText() + "_.conf_curr" );

        if ( ListBox_GameProfile->currentText().isEmpty() ) {
                KMessageBox::information( this, i18n("Please select a Game Profil form List!") );
        } else {
                if ( RadioButton_DBox_Current->isChecked() ) {
                        if (CheckBox_Startmapper->isChecked()) {
                                db_start_cvs = new QProcess();

                                db_with_config_cvs.append( LineEdit_DOSBox_Binary_Current->text() );
                                db_with_config_cvs.append( "-conf" );
                                db_with_config_cvs.append( db_dir_cvs );
                                db_with_config_cvs.append( "-startmapper" );

                                db_start_cvs->setArguments( db_with_config_cvs );
                                db_start_cvs->start();

                                connect( db_start_cvs, SIGNAL( readyReadStdout() ), this, SLOT( slot_ReadStdout_DosBox_CVS() ) );
                                db_with_config_cvs.clear();
                        } else {
                                db_start_cvs = new QProcess();

                                db_with_config_cvs.append( LineEdit_DOSBox_Binary_Current->text() );
                                db_with_config_cvs.append( "-conf" );
                                db_with_config_cvs.append( db_dir_cvs );

                                db_start_cvs->setArguments( db_with_config_cvs );
                                db_start_cvs->start();

                                connect( db_start_cvs, SIGNAL( readyReadStdout() ), this, SLOT( slot_ReadStdout_DosBox_CVS() ) );
                                db_with_config_cvs.clear();
                        }

                } else {
                        KMessageBox::information( this, i18n("Please select a DOSBox version!") );
                }

        }
}

/********************************************************************************************************************/// CMD OUTPUT CURRENT
void DosBoxFE::slot_ReadStdout_DosBox()
{
        // this is for comandline output
        QString db_output, db_check_cvs;

        while( db_start->canReadLineStdout() ) {
                db_output = db_start->readLineStdout();
                LBox_OutPut->insertItem( i18n("dosbox cmd output -> ") + db_output.mid( db_output.find( ":" ) + 1));
        }

        LBox_OutPut->insertItem( i18n("dosbox cmd output -> was closed!") );
}

/****************************************************************************************************************/// this is for comandline output CVS
void DosBoxFE::slot_ReadStdout_DosBox_CVS()
{
        QString db_output_cvs;

        while( db_start_cvs->canReadLineStdout() ) {
                db_output_cvs = db_start_cvs->readLineStdout();
                LBox_OutPut->insertItem( i18n("dosbox cmd output -> ") + db_output_cvs.mid( db_output_cvs.find( ":" ) + 1));
        }

        LBox_OutPut->insertItem( i18n("dosbox cmd output -> was closed!") );
}

/****************************************************************************************************************/// Choose Directory for mount into DOSBox
void DosBoxFE::slot_Choose_Mount_Directory()
{
        QFileDialog* fd = new QFileDialog( this, i18n("Select a Diectory"), TRUE );
        QString fileName;

        fd->setMode( QFileDialog::DirectoryOnly );
        fd->setDir( QDir::homeDirPath() );
        fd->setShowHiddenFiles(false);
        fd->show();

        if ( fd->exec() == QDialog::Accepted )
                fileName = fd->selectedFile();

        if ( CheckBox_Mount_As_CDRM->isChecked() ) {
                LineEdit_Directory_DOSBox->setText( "mount d " + fileName + " -t cdrom" );
        } else {
                LineEdit_Directory_DOSBox->setText( "mount c " + fileName );
        }

        if( CheckBox_Mount_As_Floppy->isChecked() ) {
                LineEdit_Directory_DOSBox->setText( "mount a " + fileName + " -t floppy"  );
        }
}

/***********************************************************************************************************/// Add entry from Linedit into Autoexec Listbox
void DosBoxFE::slot_Add_Directory()
{
        if( LineEdit_Directory_DOSBox->text().isEmpty() ) {
                KMessageBox::information( this, i18n("Please select a Folder for mount into the DOSBox!") );
        } else {
                ListBox_MountDir->insertItem( LineEdit_Directory_DOSBox->text() );
        }

        LineEdit_Directory_DOSBox->setText( "" );
}

/***********************************************************************************************************/// Choose snapschot dir
void DosBoxFE::slot_Choose_SnapDir()
{
        QFileDialog* fd = new QFileDialog( this, i18n("Select a Snapshoot Diectory"), TRUE );
        QString fileName;

        fd->setMode( QFileDialog::DirectoryOnly );
        fd->setDir( QDir::homeDirPath() );
        fd->setShowHiddenFiles(false);
        fd->show();

        if ( fd->exec() == QDialog::Accepted )
                fileName = fd->selectedFile();
        LineEdit_Render_SnapDir->setText(fileName);
}

/***********************************************************************************************************/// Download Update
void DosBoxFE::slot_Download_Update()
{
        DosBoxFE_UpdateWizard *wizard = new DosBoxFE_UpdateWizard();
        wizard->show();


        //The src code comming soon :)
}

/***********************************************************************************************************/// Verify Update
void DosBoxFE::slot_Verify_Update()
{
        QString dir, file;
        dir = QDir::homeDirPath();
        dir.append( "/.kde/share/config" );

        file = QDir::homeDirPath();
        file.append( "/.kde/share/config/dbfevrc" );

        download_update = new QUrlOperator();
        download_update->copy( QString( Update_File_TXT ), dir );

        connect( download_update,SIGNAL(dataTransferProgress(int, int,QNetworkOperation * )),this,SLOT(slot_Update_Progress(int, int, QNetworkOperation *)));

        QFile f( file );
        if ( f.exists() ) {
                slot_ReadVersion();
        } else {
                KMessageBox::information( this, "No " + file + " availabel!\nPlease click the Verify button again ...");
        }
}

void DosBoxFE::slot_ReadVersion()
{
        QString file;
        file = QDir::homeDirPath();
        file.append( "/.kde/share/config/dbfevrc" );

        KConfig *readVersion( kapp->config() );
        QFile f( file );
        if ( f.exists() ) {
                readVersion = new KConfig( file );
                readVersion->setGroup( "DBoxFE Version" );
                TextLabel_NewVersion->setText( readVersion->readEntry( "Version", TextLabel_NewVersion->text() ));
        }

        // a litle code for verify versions string, a better code come later ...
        if ( TextLabel_NewVersion->text() <= TextLabel_AvailableVersion->text() ) {
                btt_Download_Update->setEnabled(true);
        } else {
                btt_Download_Update->setEnabled(true);
        }
}

void DosBoxFE::slot_Update_Progress( int bytesDone, int bytesTotal, QNetworkOperation * )
{
        pBar_Status_Update->setTotalSteps( bytesTotal );
        pBar_Status_Update->setProgress( bytesDone );
        pBar_Status_Update->reset();
}

/********************************************************************************************************************/// Choose DOSBox binary Current
void DosBoxFE::slot_Choose_DBox_Binary_Curr()
{
        QFileDialog* fd = new QFileDialog( this, i18n("Select a Stabel Directory for DOSBox"), TRUE );
        QString fileName;

        fd->setMode( QFileDialog::ExistingFile );
        fd->setFilter( i18n("Stabel version of DOSBox (dosbox)") );
        fd->setDir( QDir::homeDirPath() );
        fd->setShowHiddenFiles(false);
        fd->show();

        if ( fd->exec() == QDialog::Accepted )
                fileName = fd->selectedFile();
        LineEdit_DOSBox_Binary_Current->setText(fileName);

        /*// this check the version of dosbox
        db_version = new QProcess( this );
        dbox_version.append( "dosbox" );
        dbox_version.append( "--version" );

        db_version->setArguments( rm_dbfevrc );
        db_version->start();

        connect( db_version, SIGNAL( readyReadStdout() ), this, SLOT( slot_ReadStdout_DosBox_Version() ) );
        rm_dbfevrc.clear();*/
}

void DosBoxFE::slot_ReadStdout_DosBox_Version()
{
        /*QString db_binary_version;
        // DOSBox version
        while( db_version->canReadLineStdout() )
        {
            db_binary_version = db_version->readLineStdout();
            kdDebug() << "dosbox cmd output -> " << db_binary_version << endl;
        }  
        TextLabel_DBox_Version_Check->setText( db_binary_version );*/
}

/********************************************************************************************************************/// Choose DOSBox binary CVS
void DosBoxFE::slot_Choose_DBox_Binary_CVS()
{
        QFileDialog* fd = new QFileDialog( this, i18n("Select a CVS Directory for DOSBox"), TRUE );
        QString fileName;

        fd->setMode( QFileDialog::ExistingFile );
        fd->setFilter( i18n("CVS version of DOSBox (dosbox)") );
        fd->setDir( QDir::homeDirPath() );
        fd->setShowHiddenFiles(false);
        fd->show();

        if ( fd->exec() == QDialog::Accepted )
                fileName = fd->selectedFile();
        LineEdit_DOSBox_Binary_CVS->setText(fileName);
}

/********************************************************************************************************************/// Context Menu for creat/remove GP
void DosBoxFE::contextMenuEvent ( QContextMenuEvent * )
{
        QPopupMenu* contextMenu = new QPopupMenu( ListBox_GameProfile );
        Q_CHECK_PTR( contextMenu );
        QLabel *caption = new QLabel( i18n("<font color=darkblue><u><b>Game Profile Menu</b></u></font>"), this );
        caption->setAlignment( Qt::AlignCenter );
        contextMenu->insertItem( caption );
        contextMenu->insertItem( i18n("remove Profile"), this, SLOT( slot_Remove_GameProfile() ) );
        contextMenu->insertItem( i18n("create Profile"), this, SLOT( slot_Add_GameProfile() ) );
        contextMenu->exec( QCursor::pos() );
        delete contextMenu;
}
/********************************************************************************************************************/// Choose Game exe
void DosBoxFE::slot_Choose_Game_File()
{
        QFileDialog* fd = new QFileDialog( this, i18n("Select a Game for DOSBox"), TRUE );
        QString fileName;

        fd->setMode( QFileDialog::ExistingFile );
        fd->setFilter( i18n("Gamefiles (*.exe *.bat *.com *.EXE *.BAT *.COM )") );
        fd->setDir( QDir::homeDirPath() );
        fd->setShowHiddenFiles(true);
        fd->show();

        if ( fd->exec() == QDialog::Accepted )
                fileName = fd->selectedFile();
        LineEdit_Game_File->setText(fileName);
}
/********************************************************************************************************************/// Add Game into the ListBox
void DosBoxFE::slot_Add_Game_File()
{
        QStringList sList_GameFiles;
        KConfig *gameFiles( kapp->config() );

        if( LineEdit_Game_File->text().isEmpty() ) {
                KMessageBox::information( this, i18n("Please select a Gamefile for mount into the DOSBox!") );
        } else {
                //Add entry from LineEdit into ListBox
                ListBox_Game_File->insertItem( LineEdit_Game_File->text() );

                //Save entry from Listbox into a special file
                for( unsigned int a = 0; a < ListBox_Game_File->count(); a++ ) {
                        sList_GameFiles.append( ListBox_Game_File->item( a )->text() );
                }

                gameFiles = new KConfig( "gamefiles.dbfe");
                gameFiles->setGroup( "Gamesfiles" );
                gameFiles->writeEntry( "Games", sList_GameFiles,':' );
                gameFiles->sync();
        }

        LineEdit_Game_File->setText( "" );
}
/*********************************************************************************************/// Remove gamefile from Listbox
void DosBoxFE::slot_Remove_Game_File()
{
        if( ListBox_Game_File->currentText().isEmpty() ) {
                KMessageBox::information( this, i18n("Please select a gamefile in order to delete it!") );
        } else {
                delete ListBox_Game_File->selectedItem();
                LBox_OutPut->insertItem( i18n("Mount Directory -> '") + ListBox_Game_File->currentText() + i18n("' was deleted") );
        }
}
/********************************************************************************************/// Change gamefile or update item
void DosBoxFE::slot_Change_Game_File()
{}

/*********************************************************************************************/// Add savegamedir into Listbox
void DosBoxFE::slot_Add_Savegame_Dir()
{
        QStringList sList_SaveGameDir;
        KConfig *saveGameDir( kapp->config() );

        if( LineEdit_Savegame_Dir->text().isEmpty() ) {
                KMessageBox::information( this, i18n("Please select a Savegamedir for mount into the DOSBox!") );
        } else {
                //Add entry from LineEdit into ListBox
                ListBox_Savegame_Dir->insertItem( LineEdit_Savegame_Dir->text() );

                //Save entry from Listbox into a special file
                for( unsigned int u = 0; u < ListBox_Savegame_Dir->count(); u++ ) {
                        sList_SaveGameDir.append( ListBox_Savegame_Dir->item( u )->text() );
                }

                saveGameDir = new KConfig( "gamefiles.dbfe");
                saveGameDir->setGroup( "Savegame Dir" );
                saveGameDir->writeEntry( "SaveGameDir", sList_SaveGameDir,':' );
                saveGameDir->sync();
        }

        LineEdit_Savegame_Dir->setText( "" );
}

/******************************************************************************************/// Remove savegamedir from ListBox
void DosBoxFE::slot_Remove_Savegame_Dir()
{
        if( ListBox_Savegame_Dir->currentText().isEmpty() ) {
                KMessageBox::information( this, i18n("Please select a savegamedir in order to delete it!") );
        } else {
                delete ListBox_Savegame_Dir->selectedItem();
                LBox_OutPut->insertItem( i18n("Savegame Directory -> '") + ListBox_Savegame_Dir->currentText() + i18n("' was deleted") );
        }
}

/*********************************************************************************************/// Choose savegamedir
void DosBoxFE::slot_Choose_Savegame_Dir()
{
        QFileDialog* fd = new QFileDialog( this, i18n("Select a Savegamedir for DOSBox"), TRUE );
        QString fileName;

        fd->setMode( QFileDialog::DirectoryOnly );
        fd->setDir( QDir::homeDirPath() );
        fd->setShowHiddenFiles(true);
        fd->show();

        if ( fd->exec() == QDialog::Accepted )
                fileName = fd->selectedFile();
        LineEdit_Savegame_Dir->setText(fileName);
}

/*********************************************************************************************/// Quit the Application
void DosBoxFE::slot_Quit()
{
        this->hide();
        delete this;
        kapp->quit();
}

/*********************************************************************************************/// Open the search gamewiard
void DosBoxFE::slot_SearchGame()
{
        DoxBoxFE_Wizard *wizard = new DoxBoxFE_Wizard(this);
        wizard->show();
}