/***************************************************************************
 *   Copyright (C) 2004-2005 by Alexander Saal                             *
 *   alex.saal@gmx.de                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "dosboxfe_updatewizard.moc"
#include "dosboxfe_updatewizard.h"

#include "update.h"       // update declaration

#include <qprogressbar.h>     // for stauts information about the download
#include <qurloperator.h>     // for download the update
#include <qnetworkprotocol.h> // for network connection
#include <qnetwork.h>         // for network connection
#include <qwizard.h>
#include <qdir.h>
#include <qfile.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qtextedit.h>
#include <qmessagebox.h>
#include <qradiobutton.h>
#include <qstringlist.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qprocess.h>
#include <qobject.h>
#include <qfiledialog.h>

#include <kconfig.h>
#include <kapp.h>
#include <kdebug.h>

DosBoxFE_UpdateWizard::DosBoxFE_UpdateWizard(QWidget* parent, const char* name, bool modal, WFlags fl)
                : DBFE_UpdateWizard(parent,name, modal,fl)
{
        // Init network, is requiered by update
        qInitNetworkProtocols();
}

DosBoxFE_UpdateWizard::~DosBoxFE_UpdateWizard()
{}

void DosBoxFE_UpdateWizard::slot_Choose_TAR()
{
        QFileDialog* fd = new QFileDialog( this, "Select TAR Programm", TRUE );
        QString fileName;

        fd->setMode( QFileDialog::ExistingFile );
        fd->setFilter( "TAR Programm (tar)");
        fd->setDir( QDir::homeDirPath() );
        fd->setShowHiddenFiles(false);
        fd->show();

        if ( fd->exec() == QDialog::Accepted )
                fileName = fd->selectedFile();
        LEdit_TAR->setText(fileName);
}

void DosBoxFE_UpdateWizard::slot_Install()
{
        TEdit_Log->clear();

        file = QDir::homeDirPath();
        file.append( "/.kde/share/config/dbfevrc" );

        // read versions string
        KConfig *readVersion( kapp->config() );
        readVersion = new KConfig( file );
        readVersion->setGroup( "DBoxFE Version" );
        str_ver = readVersion->readEntry( "Version", str_ver );

        // set directory to homeDirPath (etc. /home/alex/.dboxfe-update)
        uDir = QDir::homeDirPath();
        uDir.append( "/dboxfe-update/" );

        // create directory
        cDir.mkdir( uDir );

        // declaration for download update package
        uFile_SUSE = U_File_SUSE + str_ver + ".suse9x.rpm";
        uFile_HTTP_SUSE = Update_File_URL_SUSE + str_ver + ".suse9x.rpm";

        uFile_DEB = U_File_DEB + str_ver + ".i386.deb";
        uFile_HTTP_DEB = Update_File_URL_DEB + str_ver + ".i386.deb";

        uFile_DEB_SRC = U_File_DEB_SRC + str_ver + ".src-deb.tar.gz";
        uFile_HTTP_DEB_SRC = Update_File_URL_DEB + str_ver + ".src-deb.tar.gz";

        uFile_SRC = U_File_SRC + str_ver + ".tar.gz";
        uFile_HTTP_SRC = Update_File_URL_SRC + str_ver + ".tar.gz";

        download_suse = new QUrlOperator();
        download_suse->copy( QString( uFile_HTTP_SUSE ), uDir );

        download_debian = new QUrlOperator();
        download_debian->copy( QString( uFile_HTTP_DEB ), uDir );

        download_debian_src = new QUrlOperator();
        download_debian_src->copy( QString( uFile_HTTP_DEB_SRC ), uDir );

        download_src = new QUrlOperator();
        download_src->copy( QString( uFile_HTTP_SRC ), uDir );

        DEB = "dpkg";
        DEB_PARAM = "-i";

        RPM = "rpm";
        RPM_PARAM = "-Uhv";

        QFile f( uDir + uFile_SUSE );
        QFile f1( uDir + uFile_DEB );
        QFile f2( uDir + uFile_DEB_SRC );
        QFile f3( uDir + uFile_SRC );

        if ( f.exists() ) {
                if ( rbtn_Package_SuSE_RPM->isChecked() ) {
                        slot_Install_SuSE();
                }
        } else {
                if ( rbtn_Package_SuSE_RPM->isChecked() ) {
                        TEdit_Log->append( "Connect to update-server: ..." );
                        TEdit_Log->append( "Download SuSE 9.x RPM: ..." );
                        connect( download_suse,SIGNAL(dataTransferProgress(int, int,QNetworkOperation * )),this,SLOT(slot_Update_Progress(int, int, QNetworkOperation *)));
                }
        }

        if ( f1.exists() ) {
                if ( rbtn_Package_DEB->isChecked() ) {
                        slot_Install_DEB();
                }
        } else {
                if ( rbtn_Package_DEB->isChecked() ) {
                        TEdit_Log->append( "Connect to update-server: ..." );
                        TEdit_Log->append( "Download Debian Package: ..." );
                        connect( download_debian,SIGNAL(dataTransferProgress(int, int,QNetworkOperation * )),this,SLOT(slot_Update_Progress(int, int, QNetworkOperation *)));
                }
        }

        if ( f2.exists() ) {
                if ( rbtn_SrcPackage_DEB->isChecked() ) {
                        slot_Install_DEB_SRC();
                }
        } else {
                if ( rbtn_SrcPackage_DEB->isChecked() ) {
                        TEdit_Log->append( "Connect to update-server: ..." );
                        TEdit_Log->append( "Download Debian src Package: ..." );
                        connect( download_debian_src,SIGNAL(dataTransferProgress(int, int,QNetworkOperation * )),this,SLOT(slot_Update_Progress(int, int, QNetworkOperation *)));
                }
        }

        if (  f3.exists() ) {
                if ( rbtn_SrcPackage->isChecked() ) {
                        slot_Install_SRC( uDir, uFile_SRC );
                }
        } else {
                if ( rbtn_SrcPackage->isChecked() ) {
                        TEdit_Log->append( "Connect to update-server: ..." );
                        TEdit_Log->append( "Download src package: ..." );
                        connect( download_src,SIGNAL(dataTransferProgress(int, int,QNetworkOperation * )),this,SLOT(slot_Update_Progress(int, int, QNetworkOperation *)));
                        slot_Install_SRC( uDir, uFile_SRC );
                }
        }
}

void DosBoxFE_UpdateWizard::slot_Update_Progress( int bytesDone, int bytesTotal, QNetworkOperation * )
{
        pBar_Status_Update->setTotalSteps( bytesTotal );
        pBar_Status_Update->setProgress( bytesDone );
        //pBar_Status_Update->reset();
}

// install update package
void DosBoxFE_UpdateWizard::slot_Install_SuSE()
{}

void DosBoxFE_UpdateWizard::slot_Install_DEB()
{
        TEdit_Log->append( "Install Debian Package: ..." );
}

void DosBoxFE_UpdateWizard::slot_Install_DEB_SRC()
{
        TEdit_Log->append( "Install Debian src Package: ..." );
}

void DosBoxFE_UpdateWizard::slot_Install_SRC( QString &dirName, QString &fileName )
{
        installSRC = new QProcess(this);
        inst_src_lst.append( LEdit_TAR->text() );
        inst_src_lst.append("x");
        inst_src_lst.append("f");
        inst_src_lst.append("z");
        inst_src_lst.append("v");
        inst_src_lst.append(dirName);
        inst_src_lst.append(fileName);

        installSRC->setArguments( inst_src_lst );
        connect( installSRC, SIGNAL( readyReadStdout() ), this, SLOT( slot_Install_SRC_StdOut() ) );
        if ( !installSRC->start() ) {
                // error handling
                QMessageBox::critical( this, "Fatal error", "Could not decompress archiv, please selct a tar programm.", "Ok" );
        }
        kdDebug() << inst_src_lst << endl;
        inst_src_lst.clear();
}

// read QProcess output
void DosBoxFE_UpdateWizard::slot_Install_SuSE_StdOut()
{
        TEdit_Log->append( "Install SuSE 9.x RPM: ..." );
}

void DosBoxFE_UpdateWizard::slot_Install_DEB_StdOut()
{
        TEdit_Log->append( "Install Debian Package: ..." );
}

void DosBoxFE_UpdateWizard::slot_Install_DEB_SRC_StdOut()
{
        TEdit_Log->append( "Install Debian src Package: ..." );
}

void DosBoxFE_UpdateWizard::slot_Install_SRC_StdOut()
{
        TEdit_Log->append( installSRC->readStdout() );
}